@@ -28,7 +28,6 @@ module Async
 				super(**options)
 				
 				@address = address
-				@options = options
 			end
 			
 			def to_s
@@ -36,7 +35,6 @@ module Async
 			end
 			
 			attr :address
-			attr :options
 			
 			# Bind a socket to the given address. If a block is given, the socket will be automatically closed when the block exits.
 			# @yield [Socket] the bound socket
@@ -51,11 +49,5 @@ module Async
 				Socket.connect(@address, **@options, &block)
 			end
 		end
-		
-		class Endpoint
-			def self.unix(*args, **options)
-				AddressEndpoint.new(Address.unix(*args), **options)
-			end
-		end
 	end
 end
