@@ -18,22 +18,11 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 # THE SOFTWARE.
 
+require_relative 'buffer'
+
 module Async
 	module IO
-		class BinaryString < String
-			def initialize(*args)
-				super
-				
-				force_encoding(Encoding::BINARY)
-			end
-			
-			def << string
-				super
-				
-				force_encoding(Encoding::BINARY)
-			end
-			
-			alias concat <<
-		end
+		# This is deprecated.
+		BinaryString = Buffer
 	end
 end
