@@ -26,13 +26,15 @@ require_relative 'generic'
 module Async
 	module IO
 		module Peer
+			include ::Socket::Constants
+			
 			# Is it likely that the socket is still connected?
 			# May return false positive, but won't return false negative.
 			def connected?
 				return false if @io.closed?
 				
 				# If we can wait for the socket to become readable, we know that the socket may still be open.
-				result = to_io.recv_nonblock(1, Socket::MSG_PEEK, exception: false)
+				result = to_io.recv_nonblock(1, MSG_PEEK, exception: false)
 				
 				# Either there was some data available, or we can wait to see if there is data avaialble.
 				return !result.empty? || result == :wait_readable
@@ -41,6 +43,40 @@ module Async
 				# This might be thrown by recv_nonblock.
 				return false
 			end
+			
+			# Best effort to set *_NODELAY if it makes sense. Swallows errors where possible.
+			def sync=(value)
+				super
+				
+				case self.protocol
+				when 0, IPPROTO_TCP
+					self.setsockopt(IPPROTO_TCP, TCP_NODELAY, value ? 1 : 0)
+				else
+					Async.logger.warn(self) {"Unsure how to sync=#{value} for #{self.protocol}!"}
+				end
+			rescue Errno::EINVAL
+				# On Darwin, sometimes occurs when the connection is not yet fully formed. Empirically, TCP_NODELAY is enabled despite this result.
+			rescue Errno::EOPNOTSUPP
+				# Some platforms may simply not support the operation.
+				# Async.logger.warn(self) {"Unable to set sync=#{value}!"}
+			end
+			
+			def sync
+				case self.protocol
+				when IPPROTO_TCP
+					self.getsockopt(IPPROTO_TCP, TCP_NODELAY).bool
+				else
+					true
+				end && super
+			end
+			
+			def type
+				self.local_address.socktype
+			end
+			
+			def protocol
+				self.local_address.protocol
+			end
 		end
 		
 		class BasicSocket < Generic
@@ -52,21 +88,19 @@ module Async
 			wrap_blocking_method :sendmsg, :sendmsg_nonblock
 			wrap_blocking_method :send, :sendmsg_nonblock, invert: false
 			
-			def type
-				self.local_address.socktype
-			end
-			
 			include Peer
 		end
 		
 		module Server
-			def accept_each(task: Task.current)
-				task.annotate "accepting connections #{self.local_address.inspect}"
+			def accept_each(timeout: nil, task: Task.current)
+				task.annotate "accepting connections #{self.local_address.inspect} [fd=#{self.fileno}]"
+				
+				callback = lambda do |io, address|
+					yield io, address, task: task
+				end
 				
 				while true
-					self.accept(task: task) do |io, address|
-						yield io, address, task: task
-					end
+					self.accept(timeout: timeout, task: task, &callback)
 				end
 			end
 		end
@@ -76,8 +110,7 @@ module Async
 			
 			wrap_blocking_method :recvfrom, :recvfrom_nonblock
 			
-			include ::Socket::Constants
-			
+			# @raise Errno::EAGAIN the connection failed due to the remote end being overloaded.
 			def connect(*args)
 				begin
 					async_send(:connect_nonblock, *args)
@@ -88,19 +121,20 @@ module Async
 			
 			alias connect_nonblock connect
 			
-			def accept(task: Task.current)
-				peer, address = async_send(:accept_nonblock)
+			# @param duration [Numeric] the maximum time to wait for accepting a connection, if specified.
+			def accept(timeout: nil, task: Task.current)
+				peer, address = async_send(:accept_nonblock, timeout: timeout)
 				wrapper = Socket.new(peer, task.reactor)
 				
+				wrapper.timeout = self.timeout
+				
 				return wrapper, address unless block_given?
 				
 				task.async do |task|
-					task.annotate "incoming connection #{address.inspect}"
+					task.annotate "incoming connection #{address.inspect} [fd=#{wrapper.fileno}]"
 					
 					begin
 						yield wrapper, address
-					rescue
-						Async.logger.error(self) {$!}
 					ensure
 						wrapper.close
 					end
@@ -110,12 +144,30 @@ module Async
 			alias accept_nonblock accept
 			alias sysaccept accept
 			
-			def self.build(*args, task: Task.current)
+			# Build and wrap the underlying io.
+			# @option reuse_port [Boolean] Allow this port to be bound in multiple processes.
+			# @option reuse_address [Boolean] Allow this port to be bound in multiple processes.
+			def self.build(*args, timeout: nil, reuse_address: true, reuse_port: nil, linger: nil, task: Task.current)
 				socket = wrapped_klass.new(*args)
 				
+				if reuse_address
+					socket.setsockopt(SOL_SOCKET, SO_REUSEADDR, 1)
+				end
+				
+				if reuse_port
+					socket.setsockopt(SOL_SOCKET, SO_REUSEPORT, 1)
+				end
+				
+				if linger
+					socket.setsockopt(SOL_SOCKET, SO_LINGER, linger)
+				end
+				
 				yield socket
 				
-				return self.new(socket, task.reactor)
+				wrapper = self.new(socket, task.reactor)
+				wrapper.timeout = timeout
+				
+				return wrapper
 			rescue Exception
 				socket.close if socket
 				
@@ -125,27 +177,27 @@ module Async
 			# Establish a connection to a given `remote_address`.
 			# @example
 			#  socket = Async::IO::Socket.connect(Async::IO::Address.tcp("8.8.8.8", 53))
-			# @param remote_address [Addrinfo] The remote address to connect to.
-			# @param local_address [Addrinfo] The local address to bind to before connecting.
-			# @option protcol [Integer] The socket protocol to use.
-			def self.connect(remote_address, local_address = nil, reuse_port: false, task: Task.current, **options)
+			# @param remote_address [Address] The remote address to connect to.
+			# @option local_address [Address] The local address to bind to before connecting.
+			def self.connect(remote_address, local_address: nil, task: Task.current, **options)
 				Async.logger.debug(self) {"Connecting to #{remote_address.inspect}"}
 				
 				task.annotate "connecting to #{remote_address.inspect}"
 				
 				wrapper = build(remote_address.afamily, remote_address.socktype, remote_address.protocol, **options) do |socket|
-					socket.setsockopt(::Socket::SOL_SOCKET, ::Socket::SO_REUSEADDR, reuse_port)
-					
 					if local_address
+						if defined?(IP_BIND_ADDRESS_NO_PORT)
+							# Inform the kernel (Linux 4.2+) to not reserve an ephemeral port when using bind(2) with a port number of 0. The port will later be automatically chosen at connect(2) time, in a way that allows sharing a source port as long as the 4-tuple is unique.
+							socket.setsockopt(SOL_IP, IP_BIND_ADDRESS_NO_PORT, 1)
+						end
+						
 						socket.bind(local_address.to_sockaddr)
 					end
-
-					self.new(socket, task.reactor)
 				end
 				
 				begin
 					wrapper.connect(remote_address.to_sockaddr)
-					task.annotate "connected to #{remote_address.inspect}"
+					task.annotate "connected to #{remote_address.inspect} [fd=#{wrapper.fileno}]"
 				rescue Exception
 					wrapper.close
 					raise
@@ -165,13 +217,10 @@ module Async
 			#  socket = Async::IO::Socket.bind(Async::IO::Address.tcp("0.0.0.0", 9090))
 			# @param local_address [Address] The local address to bind to.
 			# @option protocol [Integer] The socket protocol to use.
-			# @option reuse_port [Boolean] Allow this port to be bound in multiple processes.
-			def self.bind(local_address, protocol: 0, reuse_port: false, task: Task.current, **options, &block)
+			def self.bind(local_address, protocol: 0, task: Task.current, **options, &block)
 				Async.logger.debug(self) {"Binding to #{local_address.inspect}"}
 				
 				wrapper = build(local_address.afamily, local_address.socktype, protocol, **options) do |socket|
-					socket.setsockopt(::Socket::SOL_SOCKET, ::Socket::SO_REUSEADDR, true)
-					socket.setsockopt(::Socket::SOL_SOCKET, ::Socket::SO_REUSEPORT, true) if reuse_port
 					socket.bind(local_address.to_sockaddr)
 				end
 				
